'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _jsonSchemaRefParser = require('json-schema-ref-parser');

var _jsonSchemaRefParser2 = _interopRequireDefault(_jsonSchemaRefParser);

var _isNil = require('inspected/schema/is-nil');

var _isNil2 = _interopRequireDefault(_isNil);

var _isString = require('inspected/schema/is-string');

var _isString2 = _interopRequireDefault(_isString);

var _isObject = require('inspected/schema/is-object');

var _isObject2 = _interopRequireDefault(_isObject);

var _deepmerge = require('deepmerge');

var _deepmerge2 = _interopRequireDefault(_deepmerge);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const resolve = ({
  basePath,
  dereference,
  parser,
  resolver
}) => (() => {
  var _ref = _asyncToGenerator(function* (schema) {
    const refParserOptions = {
      parse: { parser },
      resolve: { custom: resolver }
    };

    if (basePath) {
      return dereference ? yield _jsonSchemaRefParser2.default.dereference(basePath, schema, refParserOptions) : yield _jsonSchemaRefParser2.default.bundle(basePath, schema, refParserOptions);
    }

    return dereference ? yield _jsonSchemaRefParser2.default.dereference(schema, refParserOptions) : yield _jsonSchemaRefParser2.default.bundle(schema, refParserOptions);
  });

  return function (_x) {
    return _ref.apply(this, arguments);
  };
})();

const parse = options => (() => {
  var _ref2 = _asyncToGenerator(function* (schema) {
    if ((0, _isNil2.default)(schema)) {
      throw new Error('No schema specified.');
    }

    if (!(0, _isString2.default)(schema) && !(0, _isObject2.default)(schema)) {
      throw new Error('Schema must be a string path or spec object.');
    }

    // TODO: validate options against schema
    if (!(0, _isNil2.default)(options) && !(0, _isObject2.default)(options)) {
      throw new Error('Options should be an object.');
    }

    const defaultOptions = {
      basePath: null,
      dereference: false,
      parser: {
        canParse: function (fileInfo) {
          return false;
        },
        parse: (() => {
          var _ref3 = _asyncToGenerator(function* (fileInfo) {});

          return function parse(_x3) {
            return _ref3.apply(this, arguments);
          };
        })()
      },
      resolver: {
        canResolve: function (fileInfo) {
          return false;
        },
        resolve: (() => {
          var _ref4 = _asyncToGenerator(function* (fileInfo) {});

          return function resolve(_x4) {
            return _ref4.apply(this, arguments);
          };
        })()
      }
    };

    const compiledOptions = Object.assign({}, defaultOptions, options);

    const parser = compiledOptions.parser ? {
      order: 1,
      canParse: function (fileInfo) {
        return compiledOptions.parser.canParse({
          path: fileInfo.url,
          extension: fileInfo.extension,
          data: fileInfo.data
        });
      },
      parse: (() => {
        var _ref5 = _asyncToGenerator(function* (fileInfo) {
          return compiledOptions.parser.parse({
            path: fileInfo.url,
            extension: fileInfo.extension,
            data: fileInfo.data
          });
        });

        return function parse(_x5) {
          return _ref5.apply(this, arguments);
        };
      })()
    } : undefined;

    const resolver = compiledOptions.resolver ? {
      order: 1,
      canRead: function (fileInfo) {
        return compiledOptions.resolver.canResolve({
          path: fileInfo.url,
          extension: fileInfo.extension
        });
      },
      read: (() => {
        var _ref6 = _asyncToGenerator(function* (fileInfo) {
          return yield compiledOptions.resolver.resolve({
            path: fileInfo.url,
            extension: fileInfo.extension
          });
        });

        return function read(_x6) {
          return _ref6.apply(this, arguments);
        };
      })()
    } : undefined;

    return yield resolve({
      basePath: compiledOptions.basePath,
      dereference: compiledOptions.dereference,
      parser,
      resolver
    })(schema);
  });

  return function (_x2) {
    return _ref2.apply(this, arguments);
  };
})();

exports.default = parse;